//*****************************************
// Photo Box with ATmega64 or Atmega2560
// CPU Clock:O16MHz Ext,Crystal 16MHz)
//   iq[Yf[^@WȂŊOUɂj
//
//   q[Yf[^́@`FbŃurohdmv
//                     ɁublvɃ`FbNƂmF鎖B
//   @@@@@@@@@uannsryv́uBoot Flash size=4096 words Boot Address=$7000v
//   @@@@@@@@@uanckdudkv́uBrown-out detection at VCC=2.7Vv
//   @@@@@@@@@urtsQbjrdkv́uExt. Crystak/Resonatir High Freq.;Start-up time:16CK+64msv
//
//
//@OLED 128x64 Control IC: SSD1306
//    H@nkdc\ihhbj
//
//@RTC RX8900(EPSON)
//
// File Name:pr_mega64_photo_box.c
//
//  PD0(SCL) : TWI-SCL
//  PD1(SDA) : TWI-SDA
//  PD2(RXD1) : infrared rec.
//  PD3(INT3) : 1 Sec Signal
//
//  PF0(ADC0) : Cds Analog Level
//
//  PG0 : LED RED
//  PG1 : SELECT SW (SW2)
//  PG2 : SET SW (SW1)
//  PG3 : LED BLU
//  PG4 : LED GRN(Not implemented)
//
//
//
//  C:\WinAVR-20100110\doc\avr-libc\avr-libc-user-manual.pdf ɊeCN[ht@C̐
//
//  2023/03/07 New  for mega328_i2c_oled Taishi
//  2023/05/13 Modify for mega328_iic_oled_rtc Taishi
//  2023/12/09 Modify for Photo Box with ATmega64 Taishi
//  2024/01/09 Modify for Photo Box with ATmega64 or ATmega2560 Taishi
//*****************************************
#include	<avr/interrupt.h>
#include	<avr/io.h>

#include	"pr_mega64_2560_photo_box_h.h"

#include	"timer_h.h"
#include	"mega_Twi_h.h"
#include	"port_bit_h.h"
#include	"oled_1306_h.h"
#include	"disp_clock_data_h.h"
#include	"Rtc_Rx8900_h.h"
#include	"set_rtc_6keta_h.h"
#include	"ad_conv_h.h"
#include	"infrared_h.h"
#include	"disp_8x48_dot_h.h"


unsigned char	gDisp_clock_mode=HMS;
unsigned char	test_string[20] = {'1','2','3','4','5','6','7','8','9','0','A',0x00};
unsigned char	rtc_err_string[20] = {'R','T','C','-','E','R','R',':',0x00};
unsigned char	nami[3] = {0x0e,0x0f,0x00};


//***************************
// Aj[V\
//@ODPbɌĂ΂
// 
//@FP̂
// ߂l
//@OAj[Vr
//@PAj[VI
//***************************
unsigned char	disp_animation(unsigned char init)
{
	static	unsigned char	ani_num_max = 72;
	static	unsigned char	ani_kazu = 0;
	unsigned char			r = 0;

	if (init == 1){
		ani_kazu = 0;
	}else{
		if(ani_kazu <= 48){
			disp_tate1_compliment(ani_kazu);
		}
		ani_kazu++;
		if(ani_num_max <= ani_kazu){
			ani_kazu = 0;
			r = 1;
		}
	}

	return(r);
}

//***************************
// ԊORf[^XCb`f[^ɕϊ
//***************************
unsigned short	trans_if_to_sw(unsigned char if_data)
{
	unsigned short	us = 0;
	static unsigned short	sw_stat = 0;

	switch(if_data){
	 case IND1_ON_NOW:			// set n߂
		us = SW1_ON_NOW;
		sw_stat |= SW1_ON;
		break;
	 case IND1_ON_CONTINUE:		// set A
		us = SW1_ON_CONTINUE;
		break;
	 case IND1_OFF_SHORT:		// set Zė
		us = SW1_OFF_SHORT;
		sw_stat &= ~(SW1_ON);
		break;
	 case IND1_OFF_LONG:		// set ė
		us = SW1_OFF_LONG;
		sw_stat &= ~(SW1_ON);
		break;
	 case IND2_ON_NOW:			// select n߂
		us = SW2_ON_NOW;
		sw_stat |= SW2_ON;
		break;
	 case IND2_ON_CONTINUE:		// select A
		us = SW2_ON_CONTINUE;
		break;
	 case IND2_OFF_SHORT:		// select Zė
		us = SW2_OFF_SHORT;
		sw_stat &= ~(SW2_ON);
		break;
	 case IND2_OFF_LONG:		// select ė
		us = SW2_OFF_LONG;
		sw_stat &= ~(SW2_ON);
		break;
	}

	us |= sw_stat;
	return(us);
}

//***************************
// ̕\[h߂
//***************************
unsigned char	next_disp_clock_mode(unsigned char sd)
{
	unsigned char	uc = 0;

	switch(gDisp_clock_mode){
	 case HMS:
		uc = YMD;
		break;
	 case YMD:
		uc = CdsTempe;
		break;
	 case CdsTempe:
		uc = HMS;
		break;
	 case WHMS:
		uc = WMD;
		break;
	 case WHM:
		uc = WMD;
		break;
	 case WMD:
		uc = YEAR;
		break;
	 case YEAR:
		uc = WCdsTempe;
		break;
	 case WCdsTempe:
		if(sd == 0){
			uc = WHM;
		}else{
			uc = WHMS;
		}
		break;
	}

	return(uc);
}

//***************************
// C
//***************************
int main(void)
{
	unsigned char	uc;
	unsigned short	us;
	unsigned char	ucinf;
	unsigned char	wd = 0;				// 1:Day of Week mode
	unsigned char	sd = 0;				// 1:SDECDOT
	unsigned char	sDisp_clock_mode;

	led_init();							//  kdc֌W̏ݒ

	led_red_on_off(1);
	delay_ms(100);

	twi_init();

#ifdef	cpu64
	led_grn_on_off(1);
#else
 #ifdef cpu2560
	led_sp1_on_off(1);
 #endif
#endif

	delay_ms(100);

	led_blu_on_off(1);

	delay_ms(1000);
	led_red_on_off(0);

#ifdef	cpu64
	led_grn_on_off(0);
#else
 #ifdef cpu2560
	led_sp1_on_off(0);
 #endif
#endif

	led_blu_on_off(0);
	delay_ms(1000);

	init_oled_G();						// Init. OLED

	led_red_on_off(1);

#ifdef	cpu64
	led_grn_on_off(1);
#else
 #ifdef cpu2560
	led_sp1_on_off(1);
 #endif
#endif

	led_blu_on_off(1);

	disp_clock_title();								// ^CǧŒ\

	led_red_on_off(0);

#ifdef	cpu64
	led_grn_on_off(0);
#else
 #ifdef cpu2560
	led_sp1_on_off(0);
 #endif
#endif

	led_blu_on_off(0);

	led_blu_on_off(1);
	uc = rtc_init();								//  qsb̏ݒ
	if(uc != 0){
		disp_clear_oled_G();							// Clear All Display
		rtc_err_string[8] = (uc +0x30);
		disp_oled_G_12x16(0*12, 1*16, rtc_err_string);	// x:0, y:1, "RTC-ERR:"
		rtc_error();								// qsbG[
	}
	delay_ms(100);
	led_blu_on_off(0);

#ifdef	cpu64
	led_grn_on_off(1);
#else
 #ifdef cpu2560
	led_sp1_on_off(1);
 #endif
#endif

	uc = rtc_read();								// RTC N ǂݏo
	if(uc != 0){
		disp_clear_oled_G();							// Clear All Display
		rtc_err_string[8] = (uc +0x30);
		disp_oled_G_12x16(0*12, 1*16, rtc_err_string);	// x:0, y:1, "RTC-ERR:"
		rtc_error();								// qsbG[
	}
	uc = rtc_tempe_read();							// RTC xf[^ǂݏo
	if(uc != 0){
		disp_clear_oled_G();							// Clear All Display
		rtc_err_string[8] = (uc +0x30);
		disp_oled_G_12x16(0*12, 1*16, rtc_err_string);	// x:0, y:1, "RTC-ERR:"
		rtc_error();								// qsbG[
	}
	delay_ms(1);

#ifdef	cpu64
	led_grn_on_off(0);
#else
 #ifdef cpu2560
	led_sp1_on_off(0);
 #endif
#endif

	ad0_init();										// `^c
	disp_8x48_dot_init();							// WSWhbg\ 

	timer2_init();									// Timer2 Initial
	sw_init();										// XCb`֌W̏ݒ
	infrared_init();								// ԊORfq Initial

// *************************************************

	if(check_s1_sw_on() == 1){						// SET
		if(check_s2_sw_on() == 1){					// SELECT
			check_8x48_dot_static();				// 8x48 dot Dispaly X^eBbN`FbN
		}											//  ̊֐͔ɉiɓJԂ
	}
// *************************************************

	disp_8x48_G_5x7(1*8, (unsigned char *)Ver_Str);	// Version display 8x48 LED(8char)

	sei();											// 荞݉

	if(wd == 0){
		gDisp_clock_mode = HMS;						// b
	}else{
		gDisp_clock_mode = WHM;						// j
	}
	sDisp_clock_mode = gDisp_clock_mode;

	delay_ms(2000);
	disp_clear_oled_G();							// Clear All Display

	disp_clock_const();								// v̌Œ\
	clear_8x48_G();									// clear 8x48 All LED(6char)
	disp_8x48_clock_const();						// 8x48_v̌Œ\
	disp_clock_data(1);								// vf[^̕\i1=Sĕ\j
	while(1){
		us = get_action_sw();						// XCb`Ԃ
		us &= SW_MASK;								// XCb`ɂ
		ucinf = get_infrared_data();				// ԊORfqf[^
		if(us == 0){								// XCb`ĂȂ
			us = trans_if_to_sw(ucinf);				// ԊORf[^XCb`f[^ɕϊ
		}

		if(us != 0){								// ANVL鎞
			proc_no_key_timeup(1);					// L[͂L
			if((us & SW1_OFF_LONG) != 0){			// ėꂽ(set switch)
				sDisp_clock_mode = gDisp_clock_mode;
				proc_set_rtc_6keta();				// Uv킹
				gDisp_clock_mode = sDisp_clock_mode;
				clear_8x48_G();						// clear 8x48 All LED(6char)
				disp_8x48_clock_const();			// v̌Œ\
				disp_clock_data(1);					// vf[^̕\i1=Sĕ\j
			}else if((us & SW1_OFF_SHORT) != 0){	// Zėꂽ(set switch)
                if (gDisp_clock_mode == WHMS){		// jb̎
					sd = 0;
  					clear_sec_dot();				// b\NA
	              	gDisp_clock_mode = WHM;			// jɂ
                }else if (gDisp_clock_mode == WHM){		// b̎
					sd = 1;
                	gDisp_clock_mode = WHMS;		// jbɂ		
				}
			}else if((us & SW2_OFF_SHORT) != 0){	// Zėꂽ(select switch)
                gDisp_clock_mode = next_disp_clock_mode(sd);	// ̕\[hɂ
				clear_8x48_G();						// clear 8x48 All LED(6char)
				disp_8x48_clock_const();			// v̌Œ\
				disp_clock_data(1);					// vf[^̕\i1=Sĕ\j
			}else if((us & SW2_OFF_LONG) != 0){		// ėꂽ(select switch)
                if (gDisp_clock_mode == HMS){		// b̎
					wd = 1;
					if(sd == 0){
	                	gDisp_clock_mode = WHM;		// jɂ
					}else{
	                	gDisp_clock_mode = WHMS;	// jbɂ
					}
					clear_8x48_G();					// clear 8x48 All LED(6char)
					disp_8x48_clock_const();		// v̌Œ\
					disp_clock_data(1);				// vf[^̕\i1=Sĕ\j
				}else if ((gDisp_clock_mode == WHM) || (gDisp_clock_mode == WHMS)){	// j(bj̎
					wd = 0;
                	gDisp_clock_mode = HMS;			// bɂ
					clear_8x48_G();					// clear 8x48 All LED(6char)
					disp_8x48_clock_const();		// v̌Œ\
					disp_clock_data(1);				// vf[^̕\i1=Sĕ\j
				}
			}
		}

		if(rtc_1s_flg != 0){						// Pbo߂
			rtc_1s_flg = 0;
			led_blu_on_off(1);
			tim_10ms_beat = 50;
			uc = rtc_read();						// RTC N ǂݏo
			if(uc != 0){
				rtc_error();						// qsbG[
			}
			uc = rtc_tempe_read();					// RTC xf[^ǂݏo
			if(uc != 0){
				rtc_error();						// qsbG[
			}

			disp_clock_data(0);						// vf[^̕\i0=ω̕\j

			if((gRTC_min == 0)&&(gRTC_sec == 0)){		// ̎
				sDisp_clock_mode = gDisp_clock_mode;
				gDisp_clock_mode = ANIME;
				disp_animation(1);						// Aj[V
			}
		}

		if(tim_10ms_anime == 0){							// TOO
			if(gDisp_clock_mode == ANIME){					// Aj[V̎
				tim_10ms_anime = 20;						// POOr^C}[d|
				if(disp_animation(0) == 1){					// \ ߂lPŏI
					gDisp_clock_mode = sDisp_clock_mode;
					clear_8x48_G();							// clear 8x48 All LED(6char)
					disp_8x48_clock_const();				// v̌Œ\
					disp_clock_data(1);						// vf[^̕\i1=Sĕ\j
				}
			}
		}

 		if(tim_10ms_infrared == 0){					// ԊORfqp^C}[
			led_red_on_off(0);
		}else{
			led_red_on_off(1);
		}
 		if(tim_10ms_adget == 0){					// `^cl擾pPOrPʃ^C}[
 			tim_10ms_adget = 50;					// TOOrԊuɃZbg
			ad0_get_data();							// bf[^i邳j𕽋ςĎ擾
			if(gSet_count == 0){					// ܂Ŗ邩
				if( 550 < gAd0_ave){				// ÂȂ
					gSet_count = 3;					// Â\͂R
				}
            }else{									// ܂ňÂ
				if( gAd0_ave < 450){				// 邭Ȃ
					gSet_count = 0;					// 邢\0
				}
			}

			disp_clock_data(0);						// vf[^̕\i0=ω̕\j
		}

		if (proc_no_key_timeup(0) == 0xff){			// ԃL[͂
			if((gDisp_clock_mode != HMS) && (gDisp_clock_mode != WHM) && (gDisp_clock_mode != WHMS) && (gDisp_clock_mode != ANIME)){	// błjłȂ
				if(wd == 0){
					gDisp_clock_mode = HMS;				// b
				}else{
					if(sd == 0){
						gDisp_clock_mode = WHM;			// j
					}else{
						gDisp_clock_mode = WHMS;		// jb
					}
				}
				clear_8x48_G();							// clear 8x48 All LED(6char)
				disp_8x48_clock_const();				// v̌Œ\
				disp_clock_data(1);						// vf[^̕\i1=Sĕ\j
			}
		}

 		if(tim_10ms_beat == 0){						// C[vmFp^C}[
			led_blu_on_off(0);
		}
	}
}
